@@ -5,24 +5,22 @@ from unittest import skip
 
 from re import finditer
 
-from .helpers import create_some_fake_parts, create_a_fake_organization, create_a_fake_part_revision, \
-    create_a_fake_subpart, create_some_fake_part_classes, create_some_fake_manufacturers, create_some_fake_sellers
+from .helpers import create_some_fake_parts, create_a_fake_organization, create_a_fake_part_revision, create_user_and_organization, \
+    create_a_fake_subpart, create_some_fake_part_classes, create_some_fake_manufacturers, create_some_fake_sellers, create_a_fake_assembly
 from .models import Part, SellerPart, ManufacturerPart, Seller, PartClass, Subpart
-from .forms import PartInfoForm, PartForm, AddSubpartForm, AddSellerPartForm
+from .forms import PartInfoForm, PartFormSemiIntelligent, AddSubpartForm, AddSellerPartForm
+
+from . import constants
 
 
 class TestBOM(TransactionTestCase):
     def setUp(self):
         self.client = Client()
-        self.user = User.objects.create_user('kasper', 'kasper@McFadden.com', 'ghostpassword')
-        self.organization = create_a_fake_organization(self.user)
+        self.user, self.organization = create_user_and_organization()
         self.profile = self.user.bom_profile(organization=self.organization)
-        self.profile.role = 'A'
-        self.profile.save()
-
-    def test_home(self):
         self.client.login(username='kasper', password='ghostpassword')
 
+    def test_home(self):
         response = self.client.post(reverse('bom:home'))
         self.assertEqual(response.status_code, 200)
 
@@ -39,8 +37,6 @@ class TestBOM(TransactionTestCase):
         self.assertEqual(response.status_code, 200)
 
     def test_part_info(self):
-        self.client.login(username='kasper', password='ghostpassword')
-
         (p1, p2, p3, p4) = create_some_fake_parts(organization=self.organization)
 
         response = self.client.post(reverse('bom:part-info', kwargs={'part_id': p1.id}))
@@ -62,8 +58,6 @@ class TestBOM(TransactionTestCase):
         self.assertEqual(response.status_code, 200)
 
     def test_part_manage_bom(self):
-        self.client.login(username='kasper', password='ghostpassword')
-
         (p1, p2, p3, p4) = create_some_fake_parts(organization=self.organization)
 
         response = self.client.post(
@@ -79,50 +73,48 @@ class TestBOM(TransactionTestCase):
         self.assertEqual(response.status_code, 200)
 
     def test_part_export_bom(self):
-        self.client.login(username='kasper', password='ghostpassword')
-
         (p1, p2, p3, p4) = create_some_fake_parts(organization=self.organization)
 
         response = self.client.post(reverse('bom:part-export-bom', kwargs={'part_id': p1.id}))
         self.assertEqual(response.status_code, 200)
 
     def test_part_revision_export_bom(self):
-        self.client.login(username='kasper', password='ghostpassword')
-
         (p1, p2, p3, p4) = create_some_fake_parts(organization=self.organization)
 
         response = self.client.post(reverse('bom:part-revision-export-bom', kwargs={'part_revision_id': p1.latest().id}))
         self.assertEqual(response.status_code, 200)
 
     def test_part_revision_export_bom_flat(self):
-        self.client.login(username='kasper', password='ghostpassword')
-
         (p1, p2, p3, p4) = create_some_fake_parts(organization=self.organization)
 
         response = self.client.post(reverse('bom:part-revision-export-bom-flat', kwargs={'part_revision_id': p1.latest().id}))
         self.assertEqual(response.status_code, 200)
 
     def test_part_upload_bom(self):
-        self.client.login(username='kasper', password='ghostpassword')
         (p1, p2, p3, p4) = create_some_fake_parts(organization=self.organization)
 
-        with open('bom/test_files/test_bom.csv') as test_csv:
+        test_file = 'test_bom.csv' if self.organization.number_variation_len > 0 else 'test_bom_6_no_variations.csv'
+        with open(f'bom/test_files/{test_file}') as test_csv:
             response = self.client.post(reverse('bom:part-upload-bom', kwargs={'part_id': p2.id}), {'file': test_csv}, follow=True)
         self.assertEqual(response.status_code, 200)
 
         messages = list(response.context.get('messages'))
         for msg in messages:
-            self.assertEqual(msg.tags, "error") # Error loading 200-3333-00 via CSV because already in parent's BOM and has empty ref designators
+            self.assertEqual(msg.tags, "error")  # Error loading 200-3333-00 via CSV because already in parent's BOM and has empty ref designators
 
         subparts = p2.latest().assembly.subparts.all()
 
-        self.assertEqual(subparts[0].part_revision.part.full_part_number(), '200-3333-00')
+        expected_pn = '200-3333-00' if self.organization.number_variation_len > 0 else '200-3333'
+        self.assertEqual(subparts[0].part_revision.part.full_part_number(), expected_pn)
         self.assertEqual(subparts[0].count, 4)
-        self.assertEqual(subparts[1].part_revision.part.full_part_number(), '500-5555-00')
+
+        expected_pn = '500-5555-00' if self.organization.number_variation_len > 0 else '500-5555'
+        self.assertEqual(subparts[1].part_revision.part.full_part_number(), expected_pn)
         self.assertEqual(subparts[1].reference, 'U3, IC2, IC3')
         self.assertEqual(subparts[1].count, 3)
         self.assertEqual(subparts[1].do_not_load, False)
-        self.assertEqual(subparts[2].part_revision.part.full_part_number(), '500-5555-00')
+
+        self.assertEqual(subparts[2].part_revision.part.full_part_number(), expected_pn)
         self.assertEqual(subparts[2].reference, 'R1, R2')
         self.assertEqual(subparts[2].count, 2)
         self.assertEqual(subparts[2].do_not_load, True)
@@ -137,6 +129,8 @@ class TestBOM(TransactionTestCase):
             self.assertEqual(msg.tags, "error")
             self.assertTrue("does not exist" in str(msg.message))
 
+    def test_part_upload_bom_corner_cases(self):
+        (p1, p2, p3, p4) = create_some_fake_parts(organization=self.organization)
         with open('bom/test_files/test_bom_3_recursion.csv') as test_csv:
             response = self.client.post(reverse('bom:part-upload-bom', kwargs={'part_id': p1.id}), {'file': test_csv}, follow=True)
         self.assertEqual(response.status_code, 200)
@@ -156,16 +150,12 @@ class TestBOM(TransactionTestCase):
             self.assertTrue("revision" in str(msg.message))
 
     def test_export_part_list(self):
-        self.client.login(username='kasper', password='ghostpassword')
-
         create_some_fake_parts(organization=self.organization)
 
         response = self.client.post(reverse('bom:export-part-list'))
         self.assertEqual(response.status_code, 200)
 
     def test_create_part(self):
-        self.client.login(username='kasper', password='ghostpassword')
-
         (p1, p2, p3, p4) = create_some_fake_parts(organization=self.organization)
 
         new_part_mpn = 'STM32F401-NEW-PART'
@@ -200,11 +190,13 @@ class TestBOM(TransactionTestCase):
             'manufacturer': p1.primary_manufacturer_part.manufacturer.id,
             'number_class': p1.number_class.id,
             'number_item': '9999',
-            'number_variation': '01',
             'description': 'IC, MCU 32 Bit',
             'revision': 'A',
         }
 
+        if self.organization.number_variation_len > 0:
+            new_part_form_data['number_variation'] = '01'
+
         response = self.client.post(reverse('bom:create-part'), new_part_form_data)
         self.assertEqual(response.status_code, 302)
         self.assertTrue('/part/' in response.url)
@@ -228,11 +220,13 @@ class TestBOM(TransactionTestCase):
             'manufacturer': '',
             'number_class': p1.number_class.id,
             'number_item': '1234',
-            'number_variation': 'AZ',
             'description': 'IC, MCU 32 Bit',
             'revision': 'A',
         }
 
+        if self.organization.number_variation_len > 0:
+            new_part_form_data['number_variation'] = 'AZ'
+
         response = self.client.post(reverse('bom:create-part'), new_part_form_data)
         self.assertEqual(response.status_code, 302)
         self.assertTrue('/part/' in response.url)
@@ -273,8 +267,6 @@ class TestBOM(TransactionTestCase):
         self.assertEqual(len(occurances), 1)
 
     def test_create_part_variation(self):
-        self.client.login(username='kasper', password='ghostpassword')
-
         (p1, p2, p3, p4) = create_some_fake_parts(organization=self.organization)
 
         new_part_mpn = 'STM32F401-NEW-PART'
@@ -305,8 +297,6 @@ class TestBOM(TransactionTestCase):
         self.assertTrue('already in use' in str(response.content))
 
     def test_create_part_no_manufacturer_part(self):
-        self.client.login(username='kasper', password='ghostpassword')
-
         (p1, p2, p3, p4) = create_some_fake_parts(organization=self.organization)
 
         new_part_mpn = 'STM32F401-NEW-PART'
@@ -315,7 +305,6 @@ class TestBOM(TransactionTestCase):
             'manufacturer': '',
             'number_class': p1.number_class.id,
             'number_item': '2000',
-            'number_variation': '01',
             'configuration': 'W',
             'description': 'IC, MCU 32 Bit',
             'revision': 'A',
@@ -323,13 +312,16 @@ class TestBOM(TransactionTestCase):
             'value': ''
         }
 
+        number_variation = None
+        if self.organization.number_variation_len > 0:
+            number_variation = '01'
+            new_part_form_data['number_variation'] = number_variation
+
         response = self.client.post(reverse('bom:create-part'), new_part_form_data)
-        part = Part.objects.filter(number_class=p1.number_class.id, number_item='2000', number_variation='01').first()
+        part = Part.objects.get(number_class=p1.number_class.id, number_item='2000', number_variation=number_variation)
         self.assertEqual(len(part.manufacturer_parts()), 0)
 
     def test_part_edit(self):
-        self.client.login(username='kasper', password='ghostpassword')
-
         (p1, p2, p3, p4) = create_some_fake_parts(organization=self.organization)
 
         response = self.client.get(reverse('bom:part-edit', kwargs={'part_id': p1.id}))
@@ -345,16 +337,12 @@ class TestBOM(TransactionTestCase):
         self.assertEqual(response.status_code, 302)
 
     def test_part_delete(self):
-        self.client.login(username='kasper', password='ghostpassword')
-
         (p1, p2, p3, p4) = create_some_fake_parts(organization=self.organization)
 
         response = self.client.post(reverse('bom:part-delete', kwargs={'part_id': p1.id}))
         self.assertEqual(response.status_code, 302)
 
     def test_add_subpart(self):
-        self.client.login(username='kasper', password='ghostpassword')
-
         (p1, p2, p3, p4) = create_some_fake_parts(organization=self.organization)
 
         # Submit with no form data
@@ -402,8 +390,6 @@ class TestBOM(TransactionTestCase):
         self.assertEqual(7, qty_p2_load)
 
     def test_add_subpart_infinite_recursion(self):
-        self.client.login(username='kasper', password='ghostpassword')
-
         (p1, p2, p3, p4) = create_some_fake_parts(organization=self.organization)
 
         # Test preventing infinite recursion
@@ -415,7 +401,7 @@ class TestBOM(TransactionTestCase):
         for m in response.wsgi_request._messages:
             if 'Added' in str(m):
                 found_error = True
-            if "Can&#39;t add a part to its self." in str(m):
+            if "Infinite recursion!" in str(m):
                 rejected_add = True
         self.assertFalse(found_error)
         self.assertTrue(rejected_add)
@@ -430,14 +416,12 @@ class TestBOM(TransactionTestCase):
         for m in response.wsgi_request._messages:
             if 'Added' in str(m):
                 found_error = True
-            if "Can&#39;t add a part to its self." in str(m):
+            if "Infinite recursion!" in str(m):
                 rejected_add = True
         self.assertFalse(found_error)
         self.assertTrue(rejected_add)
 
     def test_remove_subpart(self):
-        self.client.login(username='kasper', password='ghostpassword')
-
         (p1, p2, p3, p4) = create_some_fake_parts(organization=self.organization)
         s1 = create_a_fake_subpart(p1.latest(), count=10)
 
@@ -447,8 +431,6 @@ class TestBOM(TransactionTestCase):
         self.assertEqual(response.status_code, 302)
 
     def test_remove_all_subparts(self):
-        self.client.login(username='kasper', password='ghostpassword')
-
         (p1, p2, p3, p4) = create_some_fake_parts(organization=self.organization)
 
         part = p3
@@ -466,8 +448,6 @@ class TestBOM(TransactionTestCase):
         self.assertEqual(0, len(subparts))
 
     def test_upload_parts(self):
-        self.client.login(username='kasper', password='ghostpassword')
-
         create_some_fake_part_classes(self.organization)
 
         # part_count = Part.objects.all().count()
@@ -484,7 +464,7 @@ class TestBOM(TransactionTestCase):
         self.assertEqual(response.status_code, 302)
         found_error = False
         for m in response.wsgi_request._messages:
-            if "Part class 216 in row 2 doesn&#39;t exist. Uploading of this part skipped." in str(m):
+            if "Part class 216 in row 2" in str(m) and "Uploading of this part skipped." in str(m):
                 found_error = True
         self.assertTrue(found_error)
 
@@ -499,8 +479,6 @@ class TestBOM(TransactionTestCase):
         self.assertTrue(found_error)
 
     def test_upload_part_classes(self):
-        self.client.login(username='kasper', password='ghostpassword')
-
         # Should pass
         with open('bom/test_files/test_part_classes.csv') as test_csv:
             response = self.client.post(reverse('bom:settings'), {'file': test_csv, 'submit-part-class-upload': ''})
@@ -524,11 +502,10 @@ class TestBOM(TransactionTestCase):
         with open('bom/test_files/test_part_classes_blank_rows.csv') as test_csv:
             response = self.client.post(reverse('bom:settings'), {'file': test_csv, 'submit-part-class-upload': ''})
         self.assertEqual(response.status_code, 200)
-        self.assertTrue('Part class &#39;code&#39; in row 3 does not have a value. Uploading of this part class skipped.' in str(response.content))
-        self.assertTrue('Part class &#39;code&#39; in row 4 does not have a value. Uploading of this part class skipped.' in str(response.content))
+        self.assertTrue('in row 3 does not have a value. Uploading of this part class skipped.' in str(response.content))
+        self.assertTrue('in row 4 does not have a value. Uploading of this part class skipped.' in str(response.content))
 
     def test_upload_part_classes_parts_and_boms(self):
-        self.client.login(username='kasper', password='ghostpassword')
         self.organization.number_item_len = 5
         self.organization.save()
 
@@ -590,13 +567,10 @@ class TestBOM(TransactionTestCase):
         self.assertEqual(subparts[16].count, 1)
 
     def test_edit_user_meta(self):
-        self.client.login(username='kasper', password='ghostpassword')
         response = self.client.post(reverse('bom:user-meta-edit', kwargs={'user_meta_id': self.user.bom_profile().id}))
         self.assertEqual(response.status_code, 200)
 
     def test_add_sellerpart(self):
-        self.client.login(username='kasper', password='ghostpassword')
-
         (p1, p2, p3, p4) = create_some_fake_parts(organization=self.organization)
 
         response = self.client.get(reverse('bom:manufacturer-part-add-sellerpart',
@@ -624,8 +598,6 @@ class TestBOM(TransactionTestCase):
         self.assertTrue('/part/' in response.url)
 
     def test_sellerpart_edit(self):
-        self.client.login(username='kasper', password='ghostpassword')
-
         (p1, p2, p3, p4) = create_some_fake_parts(organization=self.organization)
 
         edit_sellerpart_form_data = {
@@ -644,16 +616,12 @@ class TestBOM(TransactionTestCase):
         self.assertEqual(response.status_code, 302)
 
     def test_sellerpart_delete(self):
-        self.client.login(username='kasper', password='ghostpassword')
-
         (p1, p2, p3, p4) = create_some_fake_parts(organization=self.organization)
         response = self.client.post(reverse('bom:sellerpart-delete', kwargs={'sellerpart_id': p1.optimal_seller().id}))
 
         self.assertEqual(response.status_code, 302)
 
     def test_add_manufacturer_part(self):
-        self.client.login(username='kasper', password='ghostpassword')
-
         (p1, p2, p3, p4) = create_some_fake_parts(organization=self.organization)
         # Test GET
         response = self.client.get(reverse('bom:part-add-manufacturer-part', kwargs={'part_id': p1.id}))
@@ -672,8 +640,6 @@ class TestBOM(TransactionTestCase):
         self.assertEqual(response.status_code, 302)
 
     def test_manufacturer_part_edit(self):
-        self.client.login(username='kasper', password='ghostpassword')
-
         (p1, p2, p3, p4) = create_some_fake_parts(organization=self.organization)
         response = self.client.post(
             reverse('bom:manufacturer-part-edit', kwargs={'manufacturer_part_id': p1.primary_manufacturer_part.id}))
@@ -712,8 +678,6 @@ class TestBOM(TransactionTestCase):
         self.assertEqual(response.status_code, 200)  # 200 means it failed validation
 
     def test_manufacturer_part_delete(self):
-        self.client.login(username='kasper', password='ghostpassword')
-
         (p1, p2, p3, p4) = create_some_fake_parts(organization=self.organization)
         response = self.client.post(
             reverse('bom:manufacturer-part-delete', kwargs={'manufacturer_part_id': p1.primary_manufacturer_part.id}))
@@ -721,8 +685,6 @@ class TestBOM(TransactionTestCase):
         self.assertEqual(response.status_code, 302)
 
     def test_part_revision_release(self):
-        self.client.login(username='kasper', password='ghostpassword')
-
         (p1, p2, p3, p4) = create_some_fake_parts(organization=self.organization)
 
         response = self.client.get(
@@ -735,8 +697,6 @@ class TestBOM(TransactionTestCase):
         self.assertEqual(response.status_code, 302)
 
     def test_part_revision_revert(self):
-        self.client.login(username='kasper', password='ghostpassword')
-
         (p1, p2, p3, p4) = create_some_fake_parts(organization=self.organization)
         response = self.client.get(
             reverse('bom:part-revision-revert', kwargs={'part_id': p1.id, 'part_revision_id': p1.latest().id}))
@@ -744,8 +704,6 @@ class TestBOM(TransactionTestCase):
         self.assertEqual(response.status_code, 302)
 
     def test_part_revision_new(self):
-        self.client.login(username='kasper', password='ghostpassword')
-
         (p1, p2, p3, p4) = create_some_fake_parts(organization=self.organization)
 
         response = self.client.get(reverse('bom:part-revision-new', kwargs={'part_id': p1.id}))
@@ -795,8 +753,6 @@ class TestBOM(TransactionTestCase):
             self.assertNotIn(nsid, previous_subpart_ids)
 
     def test_part_revision_edit(self):
-        self.client.login(username='kasper', password='ghostpassword')
-
         (p1, p2, p3, p4) = create_some_fake_parts(organization=self.organization)
         response = self.client.get(
             reverse('bom:part-revision-edit', kwargs={'part_id': p1.id, 'part_revision_id': p1.latest().id}))
@@ -818,8 +774,6 @@ class TestBOM(TransactionTestCase):
         self.assertEqual(response.status_code, 302)
 
     def test_part_revision_delete(self):
-        self.client.login(username='kasper', password='ghostpassword')
-
         (p1, p2, p3, p4) = create_some_fake_parts(organization=self.organization)
         response = self.client.post(
             reverse('bom:part-revision-delete', kwargs={'part_id': p1.id, 'part_revision_id': p1.latest().id}))
@@ -827,11 +781,255 @@ class TestBOM(TransactionTestCase):
         self.assertEqual(response.status_code, 302)
 
 
+class TestBOMIntelligent(TestBOM):
+    def setUp(self):
+        self.client = Client()
+        self.user, self.organization = create_user_and_organization()
+        self.profile = self.user.bom_profile(organization=self.organization)
+        self.organization.number_scheme = constants.NUMBER_SCHEME_INTELLIGENT
+        self.organization.save()
+        self.client.login(username='kasper', password='ghostpassword')
+
+    def test_create_part(self):
+        (p1, p2, p3, p4) = create_some_fake_parts(organization=self.organization)
+
+        new_part_mpn = 'STM32F401-NEW-PART'
+        new_part_form_data = {
+            'manufacturer_part_number': new_part_mpn,
+            'manufacturer': p1.primary_manufacturer_part.manufacturer.id,
+            'number_item': 'ABC1',
+            'configuration': 'W',
+            'description': 'IC, MCU 32 Bit',
+            'revision': 'A',
+            'attribute': '',
+            'value': ''
+        }
+
+        response = self.client.post(reverse('bom:create-part'), new_part_form_data)
+        self.assertEqual(response.status_code, 302)
+        self.assertTrue('/part/' in response.url)
+
+        try:
+            created_part_id = response.url[6:-1]
+            created_part = Part.objects.get(id=created_part_id)
+        except IndexError:
+            self.assertFalse(True, "Part maybe not created? Url looks like: {}".format(response.url))
+
+        self.assertEqual(created_part.latest().description, 'IC, MCU 32 Bit')
+        self.assertEqual(created_part.manufacturer_parts().first().manufacturer_part_number, new_part_mpn)
+
+        new_part_form_data = {
+            'manufacturer_part_number': 'STM32F401',
+            'manufacturer': p1.primary_manufacturer_part.manufacturer.id,
+            'number_item': '9999',
+            'description': 'IC, MCU 32 Bit',
+            'revision': 'A',
+        }
+
+        response = self.client.post(reverse('bom:create-part'), new_part_form_data)
+        self.assertEqual(response.status_code, 302)
+        self.assertTrue('/part/' in response.url)
+
+        new_part_form_data = {
+            'manufacturer_part_number': '',
+            'manufacturer': '',
+            'number_item': '5432',
+            'description': 'IC, MCU 32 Bit',
+            'revision': 'A',
+        }
+
+        response = self.client.post(reverse('bom:create-part'), new_part_form_data)
+        self.assertEqual(response.status_code, 302)
+        self.assertTrue('/part/' in response.url)
+
+        new_part_form_data = {
+            'manufacturer_part_number': '',
+            'manufacturer': '',
+            'number_item': '1234A',
+            'description': 'IC, MCU 32 Bit',
+            'revision': 'A',
+        }
+
+        response = self.client.post(reverse('bom:create-part'), new_part_form_data)
+        self.assertEqual(response.status_code, 302)
+        self.assertTrue('/part/' in response.url)
+
+        new_part_form_data = {
+            'manufacturer_part_number': '',
+            'manufacturer': '',
+            'number_item': '1235',
+            'description': 'IC, MCU 32 Bit',
+            'revision': 'A',
+        }
+
+        response = self.client.post(reverse('bom:create-part'), new_part_form_data)
+        self.assertEqual(response.status_code, 302)
+        self.assertTrue('/part/' in response.url)
+
+        # fail nicely
+        new_part_form_data = {
+            'manufacturer_part_number': 'ABC123',
+            'manufacturer': '',
+            'number_item': p1.number_item,
+            'description': 'IC, MCU 32 Bit',
+            'revision': 'A',
+        }
+
+        response = self.client.post(reverse('bom:create-part'), new_part_form_data)
+        self.assertEqual(response.status_code, 200)
+
+        # Make sure only one part shows up
+        response = self.client.post(reverse('bom:home'))
+        self.assertEqual(response.status_code, 200)
+
+        occurances = [m.start() for m in finditer(p1.full_part_number(), response.content.decode('utf-8'))]
+        self.assertEqual(len(occurances), 1)
+
+    @skip('Not applicable')
+    def test_create_part_variation(self):
+        pass
+
+    def test_create_part_no_manufacturer_part(self):
+        (p1, p2, p3, p4) = create_some_fake_parts(organization=self.organization)
+
+        new_part_mpn = 'STM32F401-NEW-PART'
+        new_part_form_data = {
+            'manufacturer_part_number': '',
+            'manufacturer': '',
+            'number_item': '2000',
+            'configuration': 'W',
+            'description': 'IC, MCU 32 Bit',
+            'revision': 'A',
+            'attribute': '',
+            'value': ''
+        }
+
+        response = self.client.post(reverse('bom:create-part'), new_part_form_data)
+        part = Part.objects.get(number_item='2000')
+        self.assertEqual(len(part.manufacturer_parts()), 0)
+
+    def test_part_edit(self):
+        (p1, p2, p3, p4) = create_some_fake_parts(organization=self.organization)
+
+        response = self.client.get(reverse('bom:part-edit', kwargs={'part_id': p1.id}))
+        self.assertEqual(response.status_code, 200)
+
+        edit_part_form_data = {
+            'number_item': 'HEYA',
+        }
+
+        response = self.client.post(reverse('bom:part-edit', kwargs={'part_id': p1.id}), edit_part_form_data)
+        self.assertEqual(response.status_code, 302)
+
+    def test_part_upload_bom(self):
+        (p1, p2, p3, p4) = create_some_fake_parts(organization=self.organization)
+
+        p5, _ = Part.objects.get_or_create(number_item='500-5555-00', organization=self.organization)
+        assy = create_a_fake_assembly()
+        pr5 = create_a_fake_part_revision(part=p5, assembly=assy)
+
+        p6, _ = Part.objects.get_or_create(number_item='200-3333-00', organization=self.organization)
+        assy = create_a_fake_assembly()
+        pr6 = create_a_fake_part_revision(part=p5, assembly=assy)
+
+        with open('bom/test_files/test_bom.csv') as test_csv:
+            response = self.client.post(reverse('bom:part-upload-bom', kwargs={'part_id': p2.id}), {'file': test_csv}, follow=True)
+        self.assertEqual(response.status_code, 200)
+
+        messages = list(response.context.get('messages'))
+        for msg in messages:
+            self.assertEqual(msg.tags, "error")  # Error loading 200-3333-00 via CSV because already in parent's BOM and has empty ref designators
+
+        subparts = p2.latest().assembly.subparts.all()
+
+        self.assertEqual(subparts[0].part_revision.part.full_part_number(), '3333')
+        self.assertEqual(subparts[0].count, 4)
+        self.assertEqual(subparts[1].part_revision.part.full_part_number(), '500-5555-00')
+        self.assertEqual(subparts[1].reference, 'U3, IC2, IC3')
+        self.assertEqual(subparts[1].count, 3)
+        self.assertEqual(subparts[1].do_not_load, False)
+        self.assertEqual(subparts[2].part_revision.part.full_part_number(), '500-5555-00')
+        self.assertEqual(subparts[2].reference, 'R1, R2')
+        self.assertEqual(subparts[2].count, 2)
+        self.assertEqual(subparts[2].do_not_load, True)
+
+    def test_upload_parts(self):
+        create_some_fake_part_classes(self.organization)
+
+        # part_count = Part.objects.all().count()
+        # Should pass
+        with open('bom/test_files/test_new_parts_5_intelligent.csv') as test_csv:
+            response = self.client.post(reverse('bom:upload-parts'), {'file': test_csv})
+        self.assertEqual(response.status_code, 302)
+        new_part_count = Part.objects.all().count()
+        self.assertEqual(new_part_count, 4)
+
+    @skip('not applicable')
+    def test_upload_part_classes(self):
+        pass
+
+    @skip('not applicable')
+    def test_part_upload_bom_corner_cases(self):
+        pass
+
+    def test_upload_part_classes_parts_and_boms(self):
+        # TODO: Make this more robust
+        self.organization.number_item_len = 5
+        self.organization.save()
+
+        with open('bom/test_files/test_new_parts_5_intelligent.csv') as test_csv:
+            response = self.client.post(reverse('bom:upload-parts'), {'file': test_csv}, follow=True)
+        messages = list(response.context.get('messages'))
+        for msg in messages:
+            self.assertEqual(msg.tags, 'info')
+
+        self.assertEqual(response.status_code, 200)
+        new_part_count = Part.objects.all().count()
+        self.assertEqual(new_part_count, 4)
+
+        pcba = Part.objects.get(number_item='DYSON-123')
+
+        with open('bom/test_files/test_bom_5_intelligent.csv') as test_csv:
+            response = self.client.post(reverse('bom:part-upload-bom', kwargs={'part_id': pcba.id}), {'file': test_csv}, follow=True)
+        self.assertEqual(response.status_code, 200)
+
+        messages = list(response.context.get('messages'))
+
+        for msg in messages:
+            self.assertNotEqual(msg.tags, "error")
+            self.assertEqual(msg.tags, "info")
+
+        subparts = pcba.latest().assembly.subparts.all().order_by('id')
+        self.assertEqual(subparts[0].reference, 'C1, C2, C3')
+        self.assertEqual(subparts[1].reference, 'C4, C5')
+        self.assertEqual(subparts[2].reference, None)
+
+
+class TestBOMNoVariation(TestBOM):
+    def setUp(self):
+        self.client = Client()
+        self.user, self.organization = create_user_and_organization()
+        self.profile = self.user.bom_profile(organization=self.organization)
+        self.organization.number_variation_len = 0
+        self.organization.save()
+        self.client.login(username='kasper', password='ghostpassword')
+
+    @skip('not applicable')
+    def test_create_part_variation(self):
+        pass
+
+    @skip('too specific of a test case for now...')
+    def test_upload_part_classes_parts_and_boms(self):
+        pass
+
+    @skip('not applicable')
+    def test_part_upload_bom_corner_cases(self):
+        pass
+
 class TestForms(TestCase):
     def setUp(self):
         self.client = Client()
-        self.user = User.objects.create_user(
-            'kasper', 'kasper@McFadden.com', 'ghostpassword')
+        self.user = User.objects.create_user('kasper', 'kasper@McFadden.com', 'ghostpassword')
         self.organization = create_a_fake_organization(self.user)
         self.profile = self.user.bom_profile(organization=self.organization)
 
@@ -855,7 +1053,7 @@ class TestForms(TestCase):
             'revision': 'AA'
         }
 
-        form = PartForm(data=form_data, organization=self.organization)
+        form = PartFormSemiIntelligent(data=form_data, organization=self.organization)
         self.assertTrue(form.is_valid())
 
         (m1, m2, m3) = create_some_fake_manufacturers(self.organization)
@@ -866,7 +1064,7 @@ class TestForms(TestCase):
             'revision': '1',
         }
 
-        form = PartForm(data=form_data, organization=self.organization)
+        form = PartFormSemiIntelligent(data=form_data, organization=self.organization)
         self.assertTrue(form.is_valid())
 
         new_part, created = Part.objects.get_or_create(
@@ -881,7 +1079,7 @@ class TestForms(TestCase):
     def test_part_form_blank(self):
         (pc1, pc2, pc3) = create_some_fake_part_classes(self.organization)
 
-        form = PartForm(data={}, organization=self.organization)
+        form = PartFormSemiIntelligent(data={}, organization=self.organization)
 
         self.assertFalse(form.is_valid())
         self.assertEqual(form.errors, {
@@ -932,9 +1130,9 @@ class TestJsonViews(TestCase):
         self.organization = create_a_fake_organization(self.user)
         self.profile = self.user.bom_profile(organization=self.organization)
 
-    def test_mouser_part_match_bom(self):
         self.client.login(username='kasper', password='ghostpassword')
 
+    def test_mouser_part_match_bom(self):
         (p1, p2, p3, p4) = create_some_fake_parts(organization=self.organization)
         self.assertGreaterEqual(len(p3.latest().assembly.subparts.all()), 1)
         response = self.client.get(
